describe("utility methods for twitter",
function() {
    it("should create a XmlHttpRequest",
    function() {
        expect(createXmlHttpRequest()).toBeDefined();
    });

    it("should save last response",
    function() {
        lastResponse.save(1);
        lastResponse.save(2);

        expect(lastResponse.get()).toEqual(2);
    });
});

describe("Twitter trends fetching",
function() {
    beforeEach(function() {
        this.xhr = sinon.useFakeXMLHttpRequest();
        var requests = this.requests = [];
        this.xhr.onCreate = function(xhr) {
            requests.push(xhr);
        };
        lastResponse.save('');

    });
    afterEach(function() {
        this.xhr.restore();
    });
    it("should invoke send and open on XHR",
    function() {
        var fakeXhr = {}
        fakeXhr.open = sinon.spy();
        fakeXhr.send = sinon.spy();

        fetchTrends(fakeXhr);
        expect(fakeXhr.open).toHaveBeenCalledWith('get', 'http://api.twitter.com/1/trends.json', true);
        expect(fakeXhr.open).toHaveBeenCalled();
    });
    it("should perform an xhr request",
    function() {
        fetchTrends(createXmlHttpRequest());
        expect(this.requests.length).toEqual(1);
    });

    it("should save successful last response on ready state",
    function() {
        var xhr = createXmlHttpRequest();
        fetchTrends(xhr);
        var spyReady = sinon.spy(xhr, 'onreadystatechange');
        var spy = sinon.spy(lastResponse, 'save');
        expect(this.requests.length).toEqual(1);

        this.requests[0].respond(200, {
            "Content-Type": "application/json"
        },
        response);

        expect(spyReady).toHaveBeenCalled();
        expect(lastResponse.get()).toEqual(response);
        spy.restore();
        spyReady.restore();
    });

    it("should return top two parsed trends on successful fetch",
    function() {
        lastResponse.save(response);

        var trendNames = parseTrendNames();
        expect(trendNames.length).toEqual(10);
        expect(trendNames[0]).toEqual('#endoftheworldconfessions');
        expect(trendNames[1]).toEqual('#myraptureplaylist');
        expect(trendNames[2]).toEqual('#shemakesmewanna');
        expect(trendNames[3]).toEqual('DANCE TAEMIN');
        expect(trendNames[4]).toEqual('DIVA KEY');
        expect(trendNames[5]).toEqual('RAPP MINHO');
        expect(trendNames[6]).toEqual('DUBU ONEW');
        expect(trendNames[7]).toEqual('BLING JONGHYUN');
        expect(trendNames[8]).toEqual('JUSTINdonesia');
        expect(trendNames[9]).toEqual('Jay Jay Okocha');
    });

    it("should return the top two trends on successful fetch",
    function() {
        lastResponse.save(response);

        var topTwo = parseTrendNames().returnTop(2);
        expect(topTwo[0]).toEqual('#endoftheworldconfessions');
        expect(topTwo[1]).toEqual('#myraptureplaylist');
    });

    it("should save null as last response on status different from 200",
    function() {
        var xhr = createXmlHttpRequest();
        fetchTrends(xhr);
        var spy = sinon.spy(lastResponse, 'save');

        this.requests[0].respond(404, {
            "Content-Type": "application/json"
        },
        '');
        expect(spy).toHaveBeenCalledWith(null)
        spy.restore();
    });

    it("lastResponse.get should throw an error if null is saved",
    function() {
        lastResponse.save(null);
        try {
            lastResponse.get();
            expect(true).toEqual(false);

        } catch(er) {
            expect(er.name).toEqual('NoResponseException');
        }
    });

    it("should return unable to fetch from twitter when last response was null",
    function() {
        lastResponse.save(null);

        var topTwo = parseTrendNames().returnTop(2);
        expect(topTwo.message).toEqual('Unable to connect to Twitter');

    });

});

var response = '{"as_of":"Sat, 21 May 2011 14:13:57 +0000","trends":[{"url":"http:\/\/search.twitter.com\/search?q=%23endoftheworldconfessions","name":"#endoftheworldconfessions"},{"url":"http:\/\/search.twitter.com\/search?q=%23myraptureplaylist","name":"#myraptureplaylist"},{"url":"http:\/\/search.twitter.com\/search?q=%23shemakesmewanna","name":"#shemakesmewanna"},{"url":"http:\/\/search.twitter.com\/search?q=DANCE+TAEMIN","name":"DANCE TAEMIN"},{"url":"http:\/\/search.twitter.com\/search?q=DIVA+KEY","name":"DIVA KEY"},{"url":"http:\/\/search.twitter.com\/search?q=RAPP+MINHO","name":"RAPP MINHO"},{"url":"http:\/\/search.twitter.com\/search?q=DUBU+ONEW","name":"DUBU ONEW"},{"url":"http:\/\/search.twitter.com\/search?q=BLING+JONGHYUN","name":"BLING JONGHYUN"},{"url":"http:\/\/search.twitter.com\/search?q=JUSTINdonesia","name":"JUSTINdonesia"},{"url":"http:\/\/search.twitter.com\/search?q=Jay+Jay+Okocha","name":"Jay Jay Okocha"}]}';