
describe("Twitter trends", function() {
	beforeEach(function() {
	  	lastResponse = saveLastResponse('');
	});
	it("should return a new XHR object", function() {
	  	var a = createXmlHttpRequest();
		expect(a).toBeTruthy();
	});
	it("should send a ajax call to api.twitter.com", function() {
		var xhr = createXmlHttpRequest();
		var xhrOpen = spyOn(xhr, 'open');
		var xhrSend = spyOn(xhr, 'send');
		
		fetchTrends(xhr);

		expect(xhrSend).toHaveBeenCalled();	
		expect(xhrOpen).toHaveBeenCalledWith('GET', 'http://api.twitter.com/1/trends.json', true);
				
	});
	
	it("should test that last call returns the last call", function() {		
	  	expect(lastResponse.get()).toEqual('');
	
		lastResponse = saveLastResponse('a');
		lastResponse = saveLastResponse('b');
		expect(lastResponse.get()).toEqual('b');
	});
	
	it("expect printTrends to return top two trends on successful last call", function() {
	  	lastResponse = saveLastResponse(response);
		var topTwo = printTrends();
		
		expect(topTwo[0]).toEqual("#endoftheworldconfessions");
		expect(topTwo[1]).toEqual('#myraptureplaylist');
	});
	
	it("should call lastResponse(null) when xhrstatus dont equal 200", function() {
		var fakexhr = createFakeXhr(404);
		fetchTrends(fakexhr);
		spyOn(window, 'saveLastResponse');
		fakexhr.onreadystatechange();
		
		expect(window.saveLastResponse).toHaveBeenCalled();
	});
	
	it("should call lastResponse(responseText) when xhrstatus equal 200", function() {
		var fakexhr = createFakeXhr(200);
		fakexhr.responseText = "Hello!"
		fetchTrends(fakexhr);
		spyOn(window, 'saveLastResponse');
		fakexhr.onreadystatechange();
		
		expect(window.saveLastResponse).toHaveBeenCalledWith('Hello!');
	});
	
	it("expects lastResponse.get to throw an error when saveLastResponse is called with null", function() {
	  	lastResponse = saveLastResponse(null);

		var er = {name: 'EmptyResponseException' , message: 'Unable to retrieve a response from Twitter'};
		expect(lastResponse.get).toThrow(er);
	});
	
	it("expects printTrends to return 'Unable to retrieve a response from Twitter'", function() {
	  	var er = {name: 'EmptyResponseException' , message: 'Unable to retrieve a response from Twitter'};
		spyOn(lastResponse, 'get').andThrow(er);
		
		var topTwo = printTrends();
		expect(topTwo).toEqual('Unable to retrieve a response from Twitter');
	});
});

function createFakeXhr (status) {
	
	return {open : function () {}, send : function () {}, status : status, readyState: 4};
};

var response = '{"as_of":"Sat, 21 May 2011 14:13:57 +0000","trends":[{"url":"http:\/\/search.twitter.com\/search?q=%23endoftheworldconfessions","name":"#endoftheworldconfessions"},{"url":"http:\/\/search.twitter.com\/search?q=%23myraptureplaylist","name":"#myraptureplaylist"},{"url":"http:\/\/search.twitter.com\/search?q=%23shemakesmewanna","name":"#shemakesmewanna"},{"url":"http:\/\/search.twitter.com\/search?q=DANCE+TAEMIN","name":"DANCE TAEMIN"},{"url":"http:\/\/search.twitter.com\/search?q=DIVA+KEY","name":"DIVA KEY"},{"url":"http:\/\/search.twitter.com\/search?q=RAPP+MINHO","name":"RAPP MINHO"},{"url":"http:\/\/search.twitter.com\/search?q=DUBU+ONEW","name":"DUBU ONEW"},{"url":"http:\/\/search.twitter.com\/search?q=BLING+JONGHYUN","name":"BLING JONGHYUN"},{"url":"http:\/\/search.twitter.com\/search?q=JUSTINdonesia","name":"JUSTINdonesia"},{"url":"http:\/\/search.twitter.com\/search?q=Jay+Jay+Okocha","name":"Jay Jay Okocha"}]}';