describe 'successful Twitter trends fetching'

	before_each
  		mock_request().and_return({trends: '[{"url" : "http://search.twitter.com/search?q=%23happymothersday","name" : "#happymothersday"}, {"url" : "http://search.twitter.com/search?q=%23eliukiss", "name" : "#eliukiss"}]'}, 'application/json')
	end
	
	it 'should create an XmlHttpObject'
	  var xmlHttp = createXmlHttp();
		xmlHttp.should.not.be_null
	end
	
	it 'should include #happymothersday and #eliukiss after invoking fetchTrends'
    	fetchTrends();
		trends.trends.should.include '"name" : "#happymothersday"'
		trends.trends.should.include '"name" : "#eliukiss"'
		
	end
	
	it 'should include the top 2 names'
		var topTwo = topTwoTwitterTrends();
	  	topTwo.should.include '#happymothersday'
		topTwo.should.include '#eliukiss'
	end
end

describe 'unsucsessful Twitter trends fetching'
	
	before_each
	mock_request().and_return('', 'application/json')
	end
	
	it 'should throw error when response is empty'
		-{ fetchTrends() }.should.throw_error 'ResponseTextException'
	end
	
	it 'should return No response from Twitter'
	  	var topTwo = topTwoTwitterTrends();
		topTwo.should.equal 'No response from Twitter'
	end
end

var oldFetchTrends;
describe 'unsucessful Twitter trends fetching with own mock'
	
	before_each
	  	oldFetchTrends = fetchTrends;
		fetchTrends = function () {throw 'ResponseTextException'};
	end
	
	after_each
	  fetchTrends = oldFetchTrends;
	end
	
	it 'should return No response from Twitter'
	  	var topTwo = topTwoTwitterTrends();
		topTwo.should.equal 'No response from Twitter'
	end
end