module('utility functions in Twitter fetching');

test('create XmlHttpRequest', function() {
	var xmlHttp = createXmlHttpRequest();
	equal(typeof xmlHttp, 'object');
});

test('lastResponse saves last response', function() {
	lastResponse.save('a');
	lastResponse.save('b');

	equal(lastResponse.get(), 'b');
});

module('twitter trends successful fetch', {
	setup: function() {
		this.xhr = sinon.useFakeXMLHttpRequest();
		var requests = this.requests = [];
		this.xhr.onCreate = function (xhr) {	
			requests.push(xhr); 
			};		
	},
	teardown: function() {
		this.xhr.restore();
	}
});

test('perform XmlHttpReqest', function() {
	fetchTrends(function () {});
	equal(1, this.requests.length);
});

test('the callback is called with the correct response', function() {
	var callback = sinon.spy();
	fetchTrends(callback);

	equals(1, this.requests.length);
	this.requests[0].respond(200, { "Content-Type": "application/json" }, content);
	ok(callback.called);
	ok(callback.calledWith(content));
});


test('test content is saved on successful fetch', function() {
	this.spy(lastResponse, 'save');

	fetchTrends(lastResponse.save);
	equals(1, this.requests.length);
	this.requests[0].respond(200, { "Content-Type": "application/json" }, content);
	
	ok(lastResponse.save.calledOnce);
	ok(lastResponse.save.calledWith(content));	
});

test('throw error if fetching an undefined lastResponse', function() {
	lastResponse.save('');
	var errorIsValid = function (arg) {
		var er = { name: 'UndefinedLastResponse', message : 'Unable to connect or retrieve from Twitter. Please try fetching trends again' };		
		return arg.name == er.name && arg.message === er.message;
	};
	raises(lastResponse.get, errorIsValid);
});

test('print topTwo trends on successful fetch', function() {
	lastResponse.save(content);
	
	topTwo = printTrends();
	equal(topTwo[0], '#incollege');
	equal(topTwo[1], '#inmiddleschool');
});

test('print Unable to retrive a response from Twitter on empty response', function() {
	lastResponse.save("");
	
	topTwo = printTrends();
	equal(topTwo, 'Unable to retrive a response from Twitter');
});

test('undefined responseText triggers Unable to ... from printTrends', function() {
	fetchTrends(lastResponse.save);
	equals(1, this.requests.length);
	this.requests[0].respond(200, { "Content-Type": "application/json" }, "");
	topTwo = printTrends();

	equal(topTwo, 'Unable to retrive a response from Twitter');
});

var content = '{"trends":[{"name":"#incollege","url":"http:\/\/search.twitter.com\/search?q=%23incollege"},{"name":"#inmiddleschool","url":"http:\/\/search.twitter.com\/search?q=%23inmiddleschool"},{"name":"#butyoutweetingthough","url":"http:\/\/search.twitter.com\/search?q=%23butyoutweetingthough"},{"name":"Aldo Costa","url":"http:\/\/search.twitter.com\/search?q=%22Aldo+Costa%22"},{"name":"Somedayforitaly","url":"http:\/\/search.twitter.com\/search?q=Somedayforitaly"},{"name":"Opera Mini","url":"http:\/\/search.twitter.com\/search?q=%22Opera+Mini%22"},{"name":"Mar\u00eda Auxiliadora","url":"http:\/\/search.twitter.com\/search?q=%22Mar%C3%ADa+Auxiliadora%22"},{"name":"COWOK BASKET","url":"http:\/\/search.twitter.com\/search?q=%22COWOK+BASKET%22"},{"name":"ItalyNeedsGleeTour","url":"http:\/\/search.twitter.com\/search?q=ItalyNeedsGleeTour"},{"name":"Situaci\u00f3n Sentimental","url":"http:\/\/search.twitter.com\/search?q=%22Situaci%C3%B3n+Sentimental%22"}],"as_of":"Tue, 24 May 2011 14:35:41 +0000"}';